@@ -1,6 +1,5 @@
 /* Return error detail for failing <dlfcn.h> functions.
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2002
-	Free Software Foundation, Inc.
+   Copyright (C) 1995-2000,2002,2003,2004,2005 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -20,16 +19,29 @@
 
 #include <dlfcn.h>
 #include <libintl.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <bits/libc-lock.h>
+#include <ldsodefs.h>
+
+#if !defined SHARED && defined IS_IN_libdl
+
+char *
+dlerror (void)
+{
+  return __dlerror ();
+}
+
+#else
 
 /* Type for storing results of dynamic loading actions.  */
 struct dl_action_result
   {
     int errcode;
     int returned;
+    bool malloced;
     const char *objname;
     const char *errstring;
   };
@@ -38,6 +50,7 @@ static struct dl_action_result *static_buf;
 
 /* This is the key for the thread specific memory.  */
 static __libc_key_t key;
+__libc_once_define (static, once);
 
 /* Destructor for the thread-specific data.  */
 static void init (void);
@@ -45,11 +58,19 @@ static void free_key_mem (void *mem);
 
 
 char *
-dlerror (void)
+__dlerror (void)
 {
   char *buf = NULL;
   struct dl_action_result *result;
 
+# ifdef SHARED
+  if (__builtin_expect (_dlfcn_hook != NULL, 0))
+    return _dlfcn_hook->dlerror ();
+# endif
+
+  /* If we have not yet initialized the buffer do it now.  */
+  __libc_once (once, init);
+
   /* Get error string.  */
   result = (struct dl_action_result *) __libc_getspecific (key);
   if (result == NULL)
@@ -69,9 +90,19 @@ dlerror (void)
   else if (result->errstring != NULL)
     {
       buf = (char *) result->errstring;
-      if (__asprintf (&buf, result->errcode != 0 ? "%s: %s: %s" : "%s: %s",
-		      result->objname, _(result->errstring),
-		      strerror (result->errcode)) != -1)
+      int n;
+      if (result->errcode == 0)
+	n = __asprintf (&buf, "%s%s%s",
+			result->objname,
+			result->objname[0] == '\0' ? "" : ": ",
+			_(result->errstring));
+      else
+	n = __asprintf (&buf, "%s%s%s: %s",
+			result->objname,
+			result->objname[0] == '\0' ? "" : ": ",
+			_(result->errstring),
+			strerror (result->errcode));
+      if (n != -1)
 	{
 	  /* We don't need the error string anymore.  */
 	  if (strcmp (result->errstring, "out of memory") != 0)
@@ -85,12 +116,14 @@ dlerror (void)
 
   return buf;
 }
+# ifdef SHARED
+strong_alias (__dlerror, dlerror)
+# endif
 
 int
 internal_function
 _dlerror_run (void (*operate) (void *), void *args)
 {
-  __libc_once_define (static, once);
   struct dl_action_result *result;
 
   /* If we have not yet initialized the buffer do it now.  */
@@ -123,13 +156,13 @@ _dlerror_run (void (*operate) (void *), void *args)
     {
       /* Free the error string from the last failed command.  This can
 	 happen if `dlerror' was not run after an error was found.  */
-      if (strcmp (result->errstring, "out of memory") != 0)
+      if (result->malloced)
 	free ((char *) result->errstring);
       result->errstring = NULL;
     }
 
-  result->errcode = _dl_catch_error (&result->objname, &result->errstring,
-				     operate, args);
+  result->errcode = GLRO(dl_catch_error) (&result->objname, &result->errstring,
+					  &result->malloced, operate, args);
 
   /* If no error we mark that no error string is available.  */
   result->returned = result->errstring == NULL;
@@ -149,13 +182,30 @@ init (void)
     static_buf = &last_result;
 }
 
+
+static void
+check_free (struct dl_action_result *rec)
+{
+  if (rec->errstring != NULL
+      && strcmp (rec->errstring, "out of memory") != 0)
+    {
+      /* We can free the string only if the allocation happened in the
+	 C library used by the dynamic linker.  This means, it is
+	 always the C library in the base namespave.  */
+      struct link_map *map = NULL;
+      Dl_info info;
+      if (_dl_addr (check_free, &info, &map, NULL) != 0
+	  && map != NULL && map->l_ns == 0)
+	free ((char *) rec->errstring);
+    }
+}
+
+
 static void
 __attribute__ ((destructor))
 fini (void)
 {
-  if (last_result.errstring != NULL
-      && strcmp (last_result.errstring, "out of memory") != 0)
-    free ((char *) last_result.errstring);
+  check_free (&last_result);
 }
 
 
@@ -163,12 +213,40 @@ fini (void)
 static void
 free_key_mem (void *mem)
 {
-  struct dl_action_result *result = (struct dl_action_result *) mem;
-
-  if (result->errstring != NULL
-      && strcmp (result->errstring, "out of memory") != 0)
-    free ((char *) result->errstring);
+  check_free ((struct dl_action_result *) mem);
 
   free (mem);
   __libc_setspecific (key, NULL);
 }
+
+# ifdef SHARED
+
+struct dlfcn_hook *_dlfcn_hook __attribute__((nocommon));
+libdl_hidden_data_def (_dlfcn_hook)
+
+# else
+
+static struct dlfcn_hook _dlfcn_hooks =
+  {
+    .dlopen = __dlopen,
+    .dlclose = __dlclose,
+    .dlsym = __dlsym,
+    .dlvsym = __dlvsym,
+    .dlerror = __dlerror,
+    .dladdr = __dladdr,
+    .dladdr1 = __dladdr1,
+    .dlinfo = __dlinfo,
+    .dlmopen = __dlmopen
+  };
+
+void
+__libc_register_dlfcn_hook (struct link_map *map)
+{
+  struct dlfcn_hook **hook;
+
+  hook = (struct dlfcn_hook **) __libc_dlsym_private (map, "_dlfcn_hook");
+  if (hook != NULL)
+    *hook = &_dlfcn_hooks;
+}
+# endif
+#endif
