@@ -31,7 +31,6 @@
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sysdep-cancel.h>
-#include <tls.h>
 
 
 /* Type of the constructor functions.  */
@@ -42,7 +41,6 @@ typedef void (*fini_t) (void);
 #define IDX_STILL_USED -1
 
 
-#ifdef USE_TLS
 /* Returns true we an non-empty was found.  */
 static bool
 remove_slotinfo (size_t idx, struct dtv_slotinfo_list *listp, size_t disp,
@@ -104,7 +102,6 @@ remove_slotinfo (size_t idx, struct dtv_slotinfo_list *listp, size_t disp,
   /* No non-entry in this list element.  */
   return false;
 }
-#endif
 
 
 void
@@ -138,9 +135,7 @@ _dl_close_worker (struct link_map *map)
  retry:
   dl_close_state = pending;
 
-#ifdef USE_TLS
   bool any_tls = false;
-#endif
   const unsigned int nloaded = ns->_ns_nloaded;
   char used[nloaded];
   char done[nloaded];
@@ -233,8 +228,6 @@ _dl_close_worker (struct link_map *map)
   bool do_audit = GLRO(dl_naudit) > 0 && !ns->_ns_loaded->l_auditing;
 #endif
   bool unload_any = false;
-  bool scope_mem_left = false;
-  unsigned int unload_global = 0;
   unsigned int first_loaded = ~0;
   for (unsigned int i = 0; i < nloaded; ++i)
     {
@@ -299,9 +292,6 @@ _dl_close_worker (struct link_map *map)
 	  /* We indeed have an object to remove.  */
 	  unload_any = true;
 
-	  if (imap->l_global)
-	    ++unload_global;
-
 	  /* Remember where the first dynamically loaded object is.  */
 	  if (i < first_loaded)
 	    first_loaded = i;
@@ -410,18 +400,18 @@ _dl_close_worker (struct link_map *map)
 
 	      struct r_scope_elem **old = imap->l_scope;
 
-	      imap->l_scope = newp;
+	      if (RTLD_SINGLE_THREAD_P)
+		imap->l_scope = newp;
+	      else
+		{
+		  __rtld_mrlock_change (imap->l_scope_lock);
+		  imap->l_scope = newp;
+		  __rtld_mrlock_done (imap->l_scope_lock);
+		}
 
 	      /* No user anymore, we can free it now.  */
 	      if (old != imap->l_scope_mem)
-		{
-		  if (_dl_scope_free (old))
-		    /* If _dl_scope_free used THREAD_GSCOPE_WAIT (),
-		       no need to repeat it.  */
-		    scope_mem_left = false;
-		}
-	      else
-		scope_mem_left = true;
+		free (old);
 
 	      imap->l_scope_max = new_size;
 	    }
@@ -467,51 +457,9 @@ _dl_close_worker (struct link_map *map)
   r->r_state = RT_DELETE;
   _dl_debug_state ();
 
-  if (unload_global)
-    {
-      /* Some objects are in the global scope list.  Remove them.  */
-      struct r_scope_elem *ns_msl = ns->_ns_main_searchlist;
-      unsigned int i;
-      unsigned int j = 0;
-      unsigned int cnt = ns_msl->r_nlist;
-
-      while (cnt > 0 && ns_msl->r_list[cnt - 1]->l_removed)
-	--cnt;
-
-      if (cnt + unload_global == ns_msl->r_nlist)
-	/* Speed up removing most recently added objects.  */
-	j = cnt;
-      else
-	for (i = 0; i < cnt; i++)
-	  if (ns_msl->r_list[i]->l_removed == 0)
-	    {
-	      if (i != j)
-		ns_msl->r_list[j] = ns_msl->r_list[i];
-	      j++;
-	    }
-      ns_msl->r_nlist = j;
-    }
-
-  if (!RTLD_SINGLE_THREAD_P
-      && (unload_global
-	  || scope_mem_left
-	  || (GL(dl_scope_free_list) != NULL
-	      && GL(dl_scope_free_list)->count)))
-    {
-      struct dl_scope_free_list *fsl;
-
-      THREAD_GSCOPE_WAIT ();
-      /* Now we can free any queued old scopes.  */
-      if ((fsl = GL(dl_scope_free_list)) != NULL)
-	while (fsl->count > 0)
-	  free (fsl->list[--fsl->count]);
-    }
-
-#ifdef USE_TLS
   size_t tls_free_start;
   size_t tls_free_end;
   tls_free_start = tls_free_end = NO_TLS_OFFSET;
-#endif
 
   /* Check each element of the search list to see if all references to
      it are gone.  */
@@ -524,8 +472,23 @@ _dl_close_worker (struct link_map *map)
 
 	  /* That was the last reference, and this was a dlopen-loaded
 	     object.  We can unmap it.  */
+	  if (__builtin_expect (imap->l_global, 0))
+	    {
+	      /* This object is in the global scope list.  Remove it.  */
+	      struct r_scope_elem *ns_msl = ns->_ns_main_searchlist;
+	      unsigned int cnt = ns_msl->r_nlist;
+
+	      do
+		--cnt;
+	      while (ns_msl->r_list[cnt] != imap);
+
+	      /* The object was already correctly registered.  */
+	      while (++cnt < ns_msl->r_nlist)
+		ns_msl->r_list[cnt - 1] = ns_msl->r_list[cnt];
+
+	      --ns_msl->r_nlist;
+	    }
 
-#ifdef USE_TLS
 	  /* Remove the object from the dtv slotinfo array if it uses TLS.  */
 	  if (__builtin_expect (imap->l_tls_blocksize > 0, 0))
 	    {
@@ -544,7 +507,7 @@ _dl_close_worker (struct link_map *map)
 		     this search list, going in either direction.  When the
 		     whole chunk is at the end of the used area then we can
 		     reclaim it.  */
-# if TLS_TCB_AT_TP
+#if TLS_TCB_AT_TP
 		  if (tls_free_start == NO_TLS_OFFSET
 		      || (size_t) imap->l_tls_offset == tls_free_start)
 		    {
@@ -584,7 +547,7 @@ _dl_close_worker (struct link_map *map)
 			    = tls_free_end - imap->l_tls_blocksize;
 			}
 		    }
-# elif TLS_DTV_AT_TP
+#elif TLS_DTV_AT_TP
 		  if ((size_t) imap->l_tls_offset == tls_free_end)
 		    /* Extend the contiguous chunk being reclaimed.  */
 		    tls_free_end -= imap->l_tls_blocksize;
@@ -601,12 +564,11 @@ _dl_close_worker (struct link_map *map)
 		      tls_free_start = imap->l_tls_offset;
 		      tls_free_end = tls_free_start + imap->l_tls_blocksize;
 		    }
-# else
-#  error "Either TLS_TCB_AT_TP or TLS_DTV_AT_TP must be defined"
-# endif
+#else
+# error "Either TLS_TCB_AT_TP or TLS_DTV_AT_TP must be defined"
+#endif
 		}
 	    }
-#endif
 
 	  /* We can unmap all the maps at once.  We determined the
 	     start address and length when we loaded the object and
@@ -672,7 +634,6 @@ _dl_close_worker (struct link_map *map)
 	}
     }
 
-#ifdef USE_TLS
   /* If we removed any object which uses TLS bump the generation counter.  */
   if (any_tls)
     {
@@ -682,7 +643,6 @@ _dl_close_worker (struct link_map *map)
       if (tls_free_end == GL(dl_tls_static_used))
 	GL(dl_tls_static_used) = tls_free_start;
     }
-#endif
 
 #ifdef SHARED
   /* Auditing checkpoint: we have deleted all objects.  */
@@ -742,7 +702,6 @@ _dl_close (void *_map)
 }
 
 
-#ifdef USE_TLS
 static bool __libc_freeres_fn_section
 free_slotinfo (struct dtv_slotinfo_list **elemp)
 {
@@ -769,7 +728,6 @@ free_slotinfo (struct dtv_slotinfo_list **elemp)
 
   return true;
 }
-#endif
 
 
 libc_freeres_fn (free_mem)
@@ -795,26 +753,20 @@ libc_freeres_fn (free_mem)
 	free (old);
       }
 
-#ifdef USE_TLS
   if (USE___THREAD || GL(dl_tls_dtv_slotinfo_list) != NULL)
     {
       /* Free the memory allocated for the dtv slotinfo array.  We can do
 	 this only if all modules which used this memory are unloaded.  */
-# ifdef SHARED
+#ifdef SHARED
       if (GL(dl_initial_dtv) == NULL)
 	/* There was no initial TLS setup, it was set up later when
 	   it used the normal malloc.  */
 	free_slotinfo (&GL(dl_tls_dtv_slotinfo_list));
       else
-# endif
+#endif
 	/* The first element of the list does not have to be deallocated.
 	   It was allocated in the dynamic linker (i.e., with a different
 	   malloc), and in the static library it's in .bss space.  */
 	free_slotinfo (&GL(dl_tls_dtv_slotinfo_list)->next);
     }
-#endif
-
-  void *scope_free_list = GL(dl_scope_free_list);
-  GL(dl_scope_free_list) = NULL;
-  free (scope_free_list);
 }
