@@ -1,7 +1,8 @@
-/* Conversion to and from ISO 8859-1.
-   Copyright (C) 1997-1999, 2000-2004 Free Software Foundation, Inc.
+/* Conversion to and from ISO 11548-1.
+   Copyright (C) 1997-1999, 2000-2005 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.
+   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997,
+	Samuel Thibault <samuel.thibault@ens-lyon.org>, 2005.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -22,24 +23,26 @@
 #include <stdint.h>
 
 /* Definitions used in the body of the `gconv' function.  */
-#define CHARSET_NAME		"ISO-8859-1//"
-#define FROM_LOOP		from_iso8859_1
-#define TO_LOOP			to_iso8859_1
+#define CHARSET_NAME		"ISO_11548-1//"
+#define FROM_LOOP		from_iso11548_1
+#define TO_LOOP			to_iso11548_1
 #define DEFINE_INIT		1
 #define DEFINE_FINI		1
 #define MIN_NEEDED_FROM		1
 #define MIN_NEEDED_TO		4
 
-/* First define the conversion function from ISO 8859-1 to UCS4.  */
+#define BRAILLE_UCS_BASE	0x2800
+
+/* First define the conversion function from ISO 11548-1 to UCS4.  */
 #define MIN_NEEDED_INPUT	MIN_NEEDED_FROM
 #define MIN_NEEDED_OUTPUT	MIN_NEEDED_TO
 #define LOOPFCT			FROM_LOOP
 #define BODY \
-  *((uint32_t *) outptr) = *inptr++;					      \
+  *((uint32_t *) outptr) = BRAILLE_UCS_BASE | (*inptr++);		      \
   outptr += sizeof (uint32_t);
 #define ONEBYTE_BODY \
   {									      \
-    return c;								      \
+    return BRAILLE_UCS_BASE | c;					      \
   }
 #include <iconv/loop.c>
 
@@ -51,7 +54,7 @@
 #define BODY \
   {									      \
     uint32_t ch = *((const uint32_t *) inptr);				      \
-    if (__builtin_expect (ch > 0xff, 0))				      \
+    if (__builtin_expect ((ch & 0xffffff00u) != BRAILLE_UCS_BASE, 0))	      \
       {									      \
 	UNICODE_TAG_HANDLER (ch, 4);					      \
 									      \
@@ -59,7 +62,7 @@
 	STANDARD_TO_LOOP_ERR_HANDLER (4);				      \
       }									      \
     else								      \
-      *outptr++ = (unsigned char) ch;					      \
+      *outptr++ = (unsigned char) (ch & 0xff);				      \
     inptr += 4;								      \
   }
 #define LOOP_NEED_FLAGS
