@@ -1,4 +1,4 @@
-/* Copyright (C) 1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1998, 1999, 2000, 2002, 2003 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -44,31 +44,41 @@ __BEGIN_DECLS
 /* Test whether FILDES is associated with a STREAM-based file.  */
 extern int isastream (int __fildes) __THROW;
 
-/* Receive next message from a STREAMS file.  */
+/* Receive next message from a STREAMS file.
+
+   This function is a cancellation point and therefore not marked with
+   __THROW.  */
 extern int getmsg (int __fildes, struct strbuf *__restrict __ctlptr,
 		   struct strbuf *__restrict __dataptr,
-		   int *__restrict __flagsp) __THROW;
+		   int *__restrict __flagsp);
 
 /* Receive next message from a STREAMS file, with *FLAGSP allowing to
-   control which message.  */
+   control which message.
+
+   This function is a cancellation point and therefore not marked with
+   __THROW.  */
 extern int getpmsg (int __fildes, struct strbuf *__restrict __ctlptr,
 		    struct strbuf *__restrict __dataptr,
-		    int *__restrict __bandp, int *__restrict __flagsp)
-     __THROW;
+		    int *__restrict __bandp, int *__restrict __flagsp);
 
 /* Perform the I/O control operation specified by REQUEST on FD.
    One argument may follow; its presence and type depend on REQUEST.
    Return value depends on REQUEST.  Usually -1 indicates error.  */
 extern int ioctl (int __fd, unsigned long int __request, ...) __THROW;
 
-/* Send a message on a STREAM.  */
-extern int putmsg (int __fildes, __const struct strbuf *__ctlptr,
-		   __const struct strbuf *__dataptr, int __flags) __THROW;
+/* Send a message on a STREAM.
 
-/* Send a message on a STREAM to the BAND.  */
+   This function is a cancellation point and therefore not marked with
+   __THROW.  */
+extern int putmsg (int __fildes, __const struct strbuf *__ctlptr,
+		   __const struct strbuf *__dataptr, int __flags);
+
+/* Send a message on a STREAM to the BAND.
+
+   This function is a cancellation point and therefore not marked with
+   __THROW.  */
 extern int putpmsg (int __fildes, __const struct strbuf *__ctlptr,
-		    __const struct strbuf *__dataptr, int __band, int __flags)
-     __THROW;
+		    __const struct strbuf *__dataptr, int __band, int __flags);
 
 /* Attach a STREAMS-based file descriptor FILDES to a file PATH in the
    file system name space.  */
